All of lore.kernel.org
 help / color / mirror / Atom feed
From: Chris Wilson <chris@chris-wilson.co.uk>
To: intel-gfx@lists.freedesktop.org
Cc: Daniel Vetter <daniel.vetter@ffwll.ch>,
	Dave Airlie <airlied@redhat.com>,
	dri-devel@lists.freedesktop.org
Subject: [PATCH 01/44] drm: Export drm_dev_init() for subclassing
Date: Wed, 15 Jun 2016 13:17:46 +0100	[thread overview]
Message-ID: <1465993109-19523-2-git-send-email-chris@chris-wilson.co.uk> (raw)
In-Reply-To: <1465993109-19523-1-git-send-email-chris@chris-wilson.co.uk>

In order to allow drivers to pack their privates and drm_device into one
struct (e.g. for subclassing), export the initialisation routines for
struct drm_device.

v2: Missed return ret. That error path had only one job to do!
v3: Cross-referencing drm_dev_init/drm_dev_alloc in kerneldoc, fix
missed error code for goto err_minors.

Signed-off-by: Chris Wilson <chris@chris-wilson.co.uk>
Cc: Dave Airlie <airlied@redhat.com>
Cc: Daniel Vetter <daniel.vetter@ffwll.ch>
Cc: dri-devel@lists.freedesktop.org
Reviewed-by: Daniel Vetter <daniel.vetter@ffwll.ch>
---
 drivers/gpu/drm/drm_drv.c | 72 +++++++++++++++++++++++++++++++++++++----------
 include/drm/drmP.h        |  3 ++
 2 files changed, 60 insertions(+), 15 deletions(-)

diff --git a/include/drm/drmP.h b/include/drm/drmP.h
index 04310cb08111..057b6ccdbe8e 100644
--- a/include/drm/drmP.h
+++ b/include/drm/drmP.h
@@ -1070,6 +1070,9 @@ extern void drm_sysfs_hotplug_event(struct drm_device *dev);
 
 struct drm_device *drm_dev_alloc(struct drm_driver *driver,
 				 struct device *parent);
+int drm_dev_init(struct drm_device *dev,
+		 struct drm_driver *driver,
+		 struct device *parent);
 void drm_dev_ref(struct drm_device *dev);
 void drm_dev_unref(struct drm_device *dev);
 int drm_dev_register(struct drm_device *dev, unsigned long flags);
diff --git a/drivers/gpu/drm/drm_drv.c b/drivers/gpu/drm/drm_drv.c
index 8b2582aeaab6..d5c458b1a3b2 100644
--- a/drivers/gpu/drm/drm_drv.c
+++ b/drivers/gpu/drm/drm_drv.c
@@ -549,11 +549,12 @@ static void drm_fs_inode_free(struct inode *inode)
 }
 
 /**
  * drm_dev_alloc - Allocate new DRM device
  * @driver: DRM driver to allocate device for
  * @parent: Parent device object
  *
- * Allocate and initialize a new DRM device. No device registration is done.
+ * Initialize a new DRM device. No device registration is done.
  * Call drm_dev_register() to advertice the device to user space and register it
  * with other core subsystems. This should be done last in the device
  * initialization sequence to make sure userspace can't access an inconsistent
@@ -564,19 +565,18 @@ static void drm_fs_inode_free(struct inode *inode)
  *
  * Note that for purely virtual devices @parent can be NULL.
  *
+ * Drivers that do not want to allocate their own device struct
+ * embedding struct &drm_device can call drm_dev_alloc() instead.
+ *
  * RETURNS:
- * Pointer to new DRM device, or NULL if out of memory.
+ * 0 on success, or error code on failure.
  */
-struct drm_device *drm_dev_alloc(struct drm_driver *driver,
-				 struct device *parent)
+int drm_dev_init(struct drm_device *dev,
+		 struct drm_driver *driver,
+		 struct device *parent)
 {
-	struct drm_device *dev;
 	int ret;
 
-	dev = kzalloc(sizeof(*dev), GFP_KERNEL);
-	if (!dev)
-		return NULL;
-
 	kref_init(&dev->ref);
 	dev->dev = parent;
 	dev->driver = driver;
@@ -617,7 +617,8 @@ struct drm_device *drm_dev_alloc(struct drm_driver *driver,
 	if (ret)
 		goto err_minors;
 
-	if (drm_ht_create(&dev->map_hash, 12))
+	ret = drm_ht_create(&dev->map_hash, 12);
+	if (ret)
 		goto err_minors;
 
 	drm_legacy_ctxbitmap_init(dev);
@@ -636,7 +637,7 @@ struct drm_device *drm_dev_alloc(struct drm_driver *driver,
 			goto err_setunique;
 	}
 
-	return dev;
+	return 0;
 
 err_setunique:
 	if (drm_core_check_feature(dev, DRIVER_GEM))
@@ -651,8 +652,49 @@ err_minors:
 	drm_fs_inode_free(dev->anon_inode);
 err_free:
 	mutex_destroy(&dev->master_mutex);
-	kfree(dev);
-	return NULL;
+	return ret;
+}
+EXPORT_SYMBOL(drm_dev_init);
+
+/**
+ * drm_dev_alloc - Allocate new DRM device
+ * @driver: DRM driver to allocate device for
+ * @parent: Parent device object
+ *
+ * Allocate and initialize a new DRM device. No device registration is done.
+ * Call drm_dev_register() to advertice the device to user space and register it
+ * with other core subsystems. This should be done last in the device
+ * initialization sequence to make sure userspace can't access an inconsistent
+ * state.
+ *
+ * The initial ref-count of the object is 1. Use drm_dev_ref() and
+ * drm_dev_unref() to take and drop further ref-counts.
+ *
+ * Note that for purely virtual devices @parent can be NULL.
+ *
+ * Drivers that wish to subclass or embed struct &drm_device into their
+ * own struct should look at using drm_dev_init() instead.
+ *
+ * RETURNS:
+ * Pointer to new DRM device, or NULL if out of memory.
+ */
+struct drm_device *drm_dev_alloc(struct drm_driver *driver,
+				 struct device *parent)
+{
+	struct drm_device *dev;
+	int ret;
+
+	dev = kzalloc(sizeof(*dev), GFP_KERNEL);
+	if (!dev)
+		return NULL;
+
+	ret = drm_dev_init(dev, driver, parent);
+	if (ret) {
+		kfree(dev);
+		return NULL;
+	}
+
+	return dev;
 }
 EXPORT_SYMBOL(drm_dev_alloc);
 
@@ -549,11 +549,12 @@ static void drm_fs_inode_free(struct inode *inode)
 
 /**
- * drm_dev_alloc - Allocate new DRM device
- * @driver: DRM driver to allocate device for
+ * drm_dev_init - Initialise new DRM device
+ * @dev: DRM device
+ * @driver: DRM driver
  * @parent: Parent device object
  *
-- 
2.8.1

_______________________________________________
Intel-gfx mailing list
Intel-gfx@lists.freedesktop.org
https://lists.freedesktop.org/mailman/listinfo/intel-gfx

  reply	other threads:[~2016-06-15 12:17 UTC|newest]

Thread overview: 63+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2016-06-15 12:17 BAT bug 95634, take 4 Chris Wilson
2016-06-15 12:17 ` Chris Wilson [this message]
2016-06-15 12:17 ` [PATCH 02/44] drm: Add a callback from connector registering Chris Wilson
2016-06-17  7:41   ` Daniel Vetter
2016-06-15 12:17 ` [PATCH 03/44] drm: Make drm_connector_register() safe against multiple calls Chris Wilson
2016-06-15 12:17 ` [PATCH 04/44] drm: Automatically unregister the connector during cleanup Chris Wilson
2016-06-17  7:44   ` Daniel Vetter
2016-06-15 12:17 ` [PATCH 05/44] drm: Automatically register/unregister all connectors Chris Wilson
2016-06-15 12:17 ` Chris Wilson
2016-06-15 12:17 ` [PATCH 06/44] drm/arc: Remove redundant calls to drm_connector_register_all() Chris Wilson
2016-06-15 12:17 ` [PATCH 07/44] drm/atmel-hlcdc: " Chris Wilson
2016-06-15 12:17 ` [PATCH 08/44] drm/hisilicon: " Chris Wilson
2016-06-15 12:17 ` [PATCH 09/44] drm/mediatek: " Chris Wilson
2016-06-15 12:17   ` Chris Wilson
2016-06-15 12:17 ` [PATCH 10/44] drm/msm: " Chris Wilson
2016-06-15 12:17 ` [PATCH 11/44] drm/rcar-du: " Chris Wilson
2016-06-15 12:17 ` [PATCH 12/44] drm: Pass the drm_dp_aux->hw_mutex to i2c for its locking Chris Wilson
2016-06-15 12:17 ` [PATCH 13/44] drm: Minimally initialise drm_dp_aux Chris Wilson
2016-06-15 12:17 ` [PATCH 14/44] drm/i915: Perform async fbdev initialisation much later Chris Wilson
2016-06-15 12:18 ` [PATCH 15/44] drm/i915: Make panel/backlight safe to setup/cleanup multiple times Chris Wilson
2016-06-16  6:34   ` Jani Nikula
2016-06-16  9:00     ` Chris Wilson
2016-06-15 12:18 ` [PATCH 16/44] drm/i915: Move panel's backlight setup next to panel init Chris Wilson
2016-06-15 12:18 ` [PATCH 17/44] drm/i915: Move intel_connector->unregister to connector->early_unregister Chris Wilson
2016-06-15 12:18 ` [PATCH 18/44] drm/i915: Move backlight unregistration to connector unregistration Chris Wilson
2016-06-15 12:18 ` [PATCH 19/44] drm/i915: Move registration actions to connector->late_register Chris Wilson
2016-06-15 12:18 ` [PATCH 20/44] drm/i915/dp: Free the drm_dp_aux along with the encoder Chris Wilson
2016-06-15 12:18 ` [PATCH 21/44] drm/i915: Move backlight registration to connector registration Chris Wilson
2016-06-15 12:18 ` [PATCH 22/44] drm/i915: Move connector registration to driver registration Chris Wilson
2016-06-15 12:18 ` [PATCH 23/44] drm/i915: Register debugfs interface last Chris Wilson
2016-06-15 12:18 ` [PATCH 24/44] drm/i915: Demidlayer driver loading Chris Wilson
2016-06-15 12:18 ` [PATCH 25/44] drm/i915: Demidlayer driver unloading Chris Wilson
2016-06-15 12:18 ` [PATCH 26/44] drm/i915: Remove redundant drm_connector_register_all() Chris Wilson
2016-06-15 12:18 ` [PATCH 27/44] drm/i915: Start exploiting drm_device subclassing Chris Wilson
2016-06-15 12:18 ` [PATCH 28/44] drm/i915: Merge i915_dma.c into i915_drv.c Chris Wilson
2016-06-15 12:18 ` [PATCH 29/44] drm/i915: Remove user controllable DRM_ERROR for i915_getparam() Chris Wilson
2016-06-15 12:33   ` Tvrtko Ursulin
2016-06-15 12:18 ` [PATCH 30/44] drm/i915: Remove user controllable DRM_ERROR for intel_get_pipe_from_crtc_id() Chris Wilson
2016-06-15 12:34   ` Tvrtko Ursulin
2016-06-15 12:18 ` [PATCH 31/44] drm/i915: Split out the PCI driver interface to i915_pci.c Chris Wilson
2016-06-15 12:18 ` [PATCH 32/44] drm/i915: Move module init/exit " Chris Wilson
2016-07-12 10:45   ` Joonas Lahtinen
2016-06-15 12:18 ` [PATCH 33/44] drm/i915: Skip idling an idle engine Chris Wilson
2016-06-15 12:18 ` [PATCH 34/44] drm/i915: Move legacy kernel context pinning to intel_ringbuffer.c Chris Wilson
2016-06-15 12:18 ` [PATCH 35/44] drm/i915: Treat kernel context as initialised Chris Wilson
2016-06-16  8:20   ` Mika Kuoppala
2016-06-16 11:00     ` Chris Wilson
2016-06-15 12:18 ` [PATCH 36/44] drm/i915: Mark all default contexts as uninitialised after context loss Chris Wilson
2016-06-15 12:18 ` [PATCH 37/44] drm/i915: No need to wait for idle on L3 remap Chris Wilson
2016-06-15 12:18 ` [PATCH 38/44] drm/i915: Split idling from forcing context switch Chris Wilson
2016-06-16  8:51   ` Mika Kuoppala
2016-06-16 10:58     ` Chris Wilson
2016-06-15 12:18 ` [PATCH 39/44] drm/i915: Only switch to default context when evicting from GGTT Chris Wilson
2016-06-15 12:18 ` [PATCH 40/44] drm/i915: Preserve current RPS frequency Chris Wilson
2016-06-16  9:15   ` Mika Kuoppala
2016-06-16 10:56     ` Chris Wilson
2016-06-15 12:18 ` [PATCH 41/44] drm/i915: Remove superfluous powersave work flushing Chris Wilson
2016-06-21 15:15   ` Chris Wilson
2016-06-15 12:18 ` [PATCH 42/44] drm/i915: Defer enabling rc6 til after we submit the first batch/context Chris Wilson
2016-06-21 15:16   ` Chris Wilson
2016-06-15 12:18 ` [PATCH 43/44] drm/i915/fbdev: Limit the global async-domain synchronization Chris Wilson
2016-06-15 12:18 ` [PATCH 44/44] drm/i915/fbdev: Flush mode configuration before lastclose Chris Wilson
2016-06-15 16:09 ` ✗ Ro.CI.BAT: warning for series starting with [01/44] drm: Export drm_dev_init() for subclassing Patchwork

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=1465993109-19523-2-git-send-email-chris@chris-wilson.co.uk \
    --to=chris@chris-wilson.co.uk \
    --cc=airlied@redhat.com \
    --cc=daniel.vetter@ffwll.ch \
    --cc=dri-devel@lists.freedesktop.org \
    --cc=intel-gfx@lists.freedesktop.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.